@@ -7,35 +7,45 @@ import enum
 import six
 import attr
 
-from cryptoparser.tls.algorithm import TlsNextProtocolName, TlsProtocolName
+from cryptodatahub.common.exception import InvalidValue
+from cryptodatahub.tls.algorithm import (
+    TlsCertificateCompressionAlgorithm,
+    TlsExtensionType,
+    TlsNamedCurve,
+    TlsNextProtocolName,
+    TlsProtocolName,
+    TlsPskKeyExchangeMode,
+    TlsTokenBindingParamater,
+)
+
 from cryptoparser.common.base import (
+    OneByteEnumParsable,
     Opaque,
     OpaqueParam,
-    TwoByteEnumComposer,
+    ProtocolVersionMajorMinorBase,
     OpaqueEnumParsable,
     TwoByteEnumParsable,
     VariantParsable,
     Vector,
+    VectorEnumCodeNumeric,
+    VectorEnumCodeString,
+    VectorParamEnumCodeNumeric,
+    VectorParamEnumCodeString,
     VectorParamNumeric,
     VectorParamParsable,
     VectorParsable,
     VectorParsableDerived,
 )
-from cryptoparser.common.exception import NotEnoughData, InvalidType, InvalidValue
+from cryptoparser.common.exception import NotEnoughData, InvalidType
 from cryptoparser.common.parse import ParsableBase, ParserBinary, ComposerBinary
+from cryptoparser.common.x509 import SignedCertificateTimestampList
 from cryptoparser.tls.algorithm import (
-    TlsNamedCurve,
-    TlsNamedCurveFactory,
     TlsECPointFormatFactory,
-    TlsSignatureAndHashAlgorithmFactory
+    TlsNamedCurveFactory,
+    TlsSignatureAndHashAlgorithmFactory,
 )
 from cryptoparser.tls.grease import TlsInvalidTypeOneByte, TlsInvalidTypeTwoByte
-from cryptoparser.tls.version import TlsProtocolVersionBase
-
-
-@attr.s(frozen=True)
-class TlsExtensionTypeParams(object):
-    code = attr.ib(validator=attr.validators.instance_of(int))
+from cryptoparser.tls.version import TlsProtocolVersion
 
 
 class TlsExtensionTypeFactory(TwoByteEnumParsable):
@@ -48,168 +58,6 @@ class TlsExtensionTypeFactory(TwoByteEnumParsable):
         raise NotImplementedError()
 
 
-class TlsExtensionType(TwoByteEnumComposer, enum.Enum):
-    SERVER_NAME = TlsExtensionTypeParams(                             # [RFC6066]
-        code=0x0000
-    )
-    MAX_FRAGMENT_LENGTH = TlsExtensionTypeParams(                     # [RFC6066]
-        code=0x0001
-    )
-    CLIENT_CERTIFICATE_URL = TlsExtensionTypeParams(                  # [RFC6066]
-        code=0x0002
-    )
-    TRUSTED_CA_KEYS = TlsExtensionTypeParams(                         # [RFC6066]
-        code=0x0003
-    )
-    TRUNCATED_HMAC = TlsExtensionTypeParams(                          # [RFC6066]
-        code=0x0004
-    )
-    STATUS_REQUEST = TlsExtensionTypeParams(                          # [RFC6066]
-        code=0x0005
-    )
-    USER_MAPPING = TlsExtensionTypeParams(                            # [RFC4681]
-        code=0x0006
-    )
-    CLIENT_AUTHZ = TlsExtensionTypeParams(                            # [RFC5878]
-        code=0x0007
-    )
-    SERVER_AUTHZ = TlsExtensionTypeParams(                            # [RFC5878]
-        code=0x0008
-    )
-    CERT_TYPE = TlsExtensionTypeParams(                               # [RFC6091]
-        code=0x0009
-    )
-    SUPPORTED_GROUPS = TlsExtensionTypeParams(                        # [RFC-IETF-TLS-RFC]
-        code=0x000a
-    )
-    EC_POINT_FORMATS = TlsExtensionTypeParams(                        # [RFC-IETF-TLS-RFC]
-        code=0x000b
-    )
-    SRP = TlsExtensionTypeParams(                                     # [RFC5054]
-        code=0x000c
-    )
-    SIGNATURE_ALGORITHMS = TlsExtensionTypeParams(                    # [RFC5246]
-        code=0x000d
-    )
-    USE_SRTP = TlsExtensionTypeParams(                                # [RFC5764]
-        code=0x000e
-    )
-    HEARTBEAT = TlsExtensionTypeParams(                               # [RFC6520]
-        code=0x000f
-    )
-    APPLICATION_LAYER_PROTOCOL_NEGOTIATION = TlsExtensionTypeParams(  # [RFC7301]
-        code=0x0010
-    )
-    STATUS_REQUEST_V2 = TlsExtensionTypeParams(                       # [RFC6961]
-        code=0x0011
-    )
-    SIGNED_CERTIFICATE_TIMESTAMP = TlsExtensionTypeParams(            # [RFC6962]
-        code=0x0012
-    )
-    CLIENT_CERTIFICATE_TYPE = TlsExtensionTypeParams(                 # [RFC7250]
-        code=0x0013
-    )
-    SERVER_CERTIFICATE_TYPE = TlsExtensionTypeParams(                 # [RFC7250]
-        code=0x0014
-    )
-    PADDING = TlsExtensionTypeParams(                                 # [RFC7685]
-        code=0x0015
-    )
-    ENCRYPT_THEN_MAC = TlsExtensionTypeParams(                        # [RFC7366]
-        code=0x0016
-    )
-    EXTENDED_MASTER_SECRET = TlsExtensionTypeParams(                  # [RFC7627]
-        code=0x0017
-    )
-    TOKEN_BINDING = TlsExtensionTypeParams(                           # [DRAFT-IETF-TOKBIND-NEGOTIATION]
-        code=0x0018
-    )
-    CACHED_INFO = TlsExtensionTypeParams(                             # [RFC7924]
-        code=0x0019
-    )
-    COMPRESS_CERTIFICATE = TlsExtensionTypeParams(                    # [RFC-ietf-tls-certificate-compression-09]
-        code=0x001b
-    )
-    RECORD_SIZE_LIMIT = TlsExtensionTypeParams(                       # [RFC8849]
-        code=0x001c
-    )
-    PWD_PROTECT = TlsExtensionTypeParams(                             # [RFC-HARKINS-TLS-DRAGONFLY-03]
-        code=0x001d
-    )
-    PWD_CLEAR = TlsExtensionTypeParams(                               # [RFC-HARKINS-TLS-DRAGONFLY-03]
-        code=0x001e
-    )
-    PASSWORD_SALT = TlsExtensionTypeParams(                           # [RFC-HARKINS-TLS-DRAGONFLY-03]
-        code=0x001f
-    )
-    TICKET_PINNING = TlsExtensionTypeParams(                          # [RFC8672]
-        code=0x0020
-    )
-    TLS_CERT_WITH_EXTERN_PSK = TlsExtensionTypeParams(                # [RFC-IETF-TLS-TLS13-CERT-WITH-EXTERN-PSK-07]
-        code=0x0021
-    )
-    SESSION_TICKET = TlsExtensionTypeParams(                          # [RFC4507]
-        code=0x0023
-    )
-    KEY_SHARE_RESERVED = TlsExtensionTypeParams(                      # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x0028
-    )
-    PRE_SHARED_KEY = TlsExtensionTypeParams(                          # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x0029
-    )
-    EARLY_DATA = TlsExtensionTypeParams(                              # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x002a
-    )
-    SUPPORTED_VERSIONS = TlsExtensionTypeParams(                      # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x002b
-    )
-    COOKIE = TlsExtensionTypeParams(                                  # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x002c
-    )
-    PSK_KEY_EXCHANGE_MODES = TlsExtensionTypeParams(                  # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x002d
-    )
-    CERTIFICATE_AUTHORITIES = TlsExtensionTypeParams(                 # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x002f
-    )
-    OID_FILTERS = TlsExtensionTypeParams(                             # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x0030
-    )
-    POST_HANDSHAKE_AUTH = TlsExtensionTypeParams(                     # [DRAFT-IETF-TLS-TLS13-20]
-        code=0x0031
-    )
-    SIGNATURE_ALGORITHMS_CERT = TlsExtensionTypeParams(               # [DRAFT-IETF-TLS-TLS13-23]
-        code=0x0032
-    )
-    KEY_SHARE = TlsExtensionTypeParams(                               # [DRAFT-IETF-TLS-TLS13-23]
-        code=0x0033
-    )
-    TRANSPARENCY_INFO = TlsExtensionTypeParams(                       # [DRAFT-IETF-TRANS-RFC6962-BIS]
-        code=0x0034
-    )
-    CONNECTION_ID = TlsExtensionTypeParams(                           # [DRAFT-IETF-TLS-DTLS-CONNECTION-ID]
-        code=0x0035
-    )
-    EXTERNAL_ID_HASH = TlsExtensionTypeParams(                        # [RFC-IETF-MMUSIC-SDP-UKS-07]
-        code=0x0037
-    )
-    EXTERNAL_SESSION_ID = TlsExtensionTypeParams(                     # [RFC-IETF-MMUSIC-SDP-UKS-07]
-        code=0x0038
-    )
-    NEXT_PROTOCOL_NEGOTIATION = TlsExtensionTypeParams(               # [DRAFT-AGL-TLS-NEXTPROTONEG-04]
-        code=0x3374
-    )
-    CHANNEL_ID = TlsExtensionTypeParams(                              # [DRAFT-BALFANZ-TLS-OBC-01]
-        code=0x7550
-    )
-    RENEGOTIATION_INFO = TlsExtensionTypeParams(                      # [DRAFT-AGL-TLS-NEXTPROTONEG-03]
-        code=0xff01
-    )
-    RECORD_HEADER = TlsExtensionTypeParams(                           # [DRAFT-FOSSATI-TLS-EXT-HEADER]
-        code=0xff03
-    )
-
-
 class TlsExtensionsBase(VectorParsable):
     @classmethod
     @abc.abstractmethod
@@ -328,7 +176,7 @@ class TlsExtensionParsed(TlsExtensionBase):
         parser.parse_parsable(name, TlsExtensionTypeFactory)
 
     def _compose_type(self, composer):
-        composer.compose_parsable(self.extension_type)
+        composer.compose_numeric_enum_coded(self.extension_type)
 
     @classmethod
     @abc.abstractmethod
@@ -381,7 +229,7 @@ class TlsServerName(Vector):
 
 
 @attr.s
-class TlsExtensionServerName(TlsExtensionParsed):
+class TlsExtensionServerNameClient(TlsExtensionParsed):
     host_name = attr.ib(validator=attr.validators.instance_of(six.string_types))
     name_type = attr.ib(validator=attr.validators.in_(TlsServerNameType), default=TlsServerNameType.HOST_NAME)
 
@@ -391,39 +239,42 @@ class TlsExtensionServerName(TlsExtensionParsed):
 
     @classmethod
     def _parse(cls, parsable):
-        parser = super(TlsExtensionServerName, cls)._parse_header(parsable)
+        parser = super(TlsExtensionServerNameClient, cls)._parse_header(parsable)
 
-        if parser['extension_length'] > 0:
-            parser.parse_numeric('server_name_list_length', 2)
-            parser.parse_numeric('server_name_type', 1, TlsServerNameType)
-            parser.parse_parsable('server_name', TlsServerName)
+        parser.parse_numeric('server_name_list_length', 2)
+        parser.parse_numeric('server_name_type', 1, TlsServerNameType)
+        parser.parse_parsable('server_name', TlsServerName)
 
-            return TlsExtensionServerName(
-                six.ensure_text(bytes(bytearray(parser['server_name'])), 'idna')
-            ), parser.parsed_length
-
-        return TlsExtensionServerName(six.ensure_text('', 'idna')), parser.parsed_length
+        return cls(
+            six.ensure_text(bytes(bytearray(parser['server_name'])), 'idna')
+        ), parser.parsed_length
 
     def compose(self):
         composer = ComposerBinary()
 
-        if self.host_name:
-            idna_encoded_host_name = six.ensure_binary(self.host_name, 'idna')
+        idna_encoded_host_name = six.ensure_binary(self.host_name, 'idna')
 
-            composer.compose_numeric(3 + len(idna_encoded_host_name), 2)
-            composer.compose_numeric(self.name_type, 1)
+        composer.compose_numeric(3 + len(idna_encoded_host_name), 2)
+        composer.compose_numeric(self.name_type, 1)
 
-            composer.compose_bytes(idna_encoded_host_name, 2)
+        composer.compose_bytes(idna_encoded_host_name, 2)
 
         header_bytes = self._compose_header(composer.composed_length)
 
         return header_bytes + composer.composed_bytes
 
 
-class TlsECPointFormatVector(VectorParsable):
+@attr.s
+class TlsExtensionServerNameServer(TlsExtensionUnusedData):
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.SERVER_NAME
+
+
+class TlsECPointFormatVector(VectorEnumCodeNumeric):
     @classmethod
     def get_param(cls):
-        return VectorParamParsable(
+        return VectorParamEnumCodeNumeric(
             item_class=TlsECPointFormatFactory,
             fallback_class=TlsInvalidTypeOneByte,
             min_byte_num=1,
@@ -460,10 +311,10 @@ class TlsExtensionECPointFormats(TlsExtensionParsed):
         return header_bytes + payload_composer.composed_bytes
 
 
-class TlsEllipticCurveVector(VectorParsable):
+class TlsEllipticCurveVector(VectorEnumCodeNumeric):
     @classmethod
     def get_param(cls):
-        return VectorParamParsable(
+        return VectorParamEnumCodeNumeric(
             item_class=TlsNamedCurveFactory,
             fallback_class=TlsInvalidTypeTwoByte,
             min_byte_num=1, max_byte_num=2 ** 16 - 1
@@ -503,7 +354,7 @@ class TlsSupportedVersionVector(VectorParsableDerived):
     @classmethod
     def get_param(cls):
         return VectorParamParsable(
-            item_class=TlsProtocolVersionBase,
+            item_class=TlsProtocolVersion,
             fallback_class=TlsInvalidTypeTwoByte,
             min_byte_num=2, max_byte_num=2 ** 8 - 2
         )
@@ -552,13 +403,13 @@ class TlsExtensionSupportedVersionsClient(TlsExtensionSupportedVersionsBase):
 
 @attr.s
 class TlsExtensionSupportedVersionsServer(TlsExtensionSupportedVersionsBase):
-    selected_version = attr.ib(validator=attr.validators.instance_of(TlsProtocolVersionBase))
+    selected_version = attr.ib(validator=attr.validators.instance_of(TlsProtocolVersion))
 
     @classmethod
     def _parse(cls, parsable):
         parser = super(TlsExtensionSupportedVersionsServer, cls)._parse_header(parsable)
 
-        parser.parse_parsable('selected_version', TlsProtocolVersionBase)
+        parser.parse_parsable('selected_version', TlsProtocolVersion)
 
         return TlsExtensionSupportedVersionsServer(parser['selected_version']), parser.parsed_length
 
@@ -572,10 +423,10 @@ class TlsExtensionSupportedVersionsServer(TlsExtensionSupportedVersionsBase):
         return header_bytes + payload_composer.composed_bytes
 
 
-class TlsSignatureAndHashAlgorithmVector(VectorParsable):
+class TlsSignatureAndHashAlgorithmVector(VectorEnumCodeNumeric):
     @classmethod
     def get_param(cls):
-        return VectorParamParsable(
+        return VectorParamEnumCodeNumeric(
             item_class=TlsSignatureAndHashAlgorithmFactory,
             fallback_class=TlsInvalidTypeTwoByte,
             min_byte_num=2, max_byte_num=2 ** 16 - 2
@@ -624,6 +475,12 @@ class TlsExtensionSignatureAlgorithmsCert(TlsExtensionSignatureAlgorithmsBase):
         return TlsExtensionType.SIGNATURE_ALGORITHMS_CERT
 
 
+class TlsExtensionDelegatedCredentials(TlsExtensionSignatureAlgorithmsBase):
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.DELEGATED_CREDENTIALS
+
+
 class TlsKeyExchangeVector(Vector):
     @classmethod
     def get_param(cls):
@@ -647,18 +504,41 @@ class TlsKeyShareEntry(ParsableBase):
     def compose(self):
         composer = ComposerBinary()
 
-        composer.compose_parsable(self.group)
+        composer.compose_numeric_enum_coded(self.group)
         composer.compose_parsable(self.key_exchange)
 
         return composer.composed_bytes
 
 
+@attr.s
+class TlsKeyShareEntryInvalidType(ParsableBase):
+    group = attr.ib(validator=attr.validators.instance_of(TlsInvalidTypeTwoByte))
+    data = attr.ib(validator=attr.validators.instance_of((bytes, bytearray)))
+
+    @classmethod
+    def _parse(cls, parsable):
+        parser = ParserBinary(parsable)
+
+        parser.parse_parsable('group', TlsInvalidTypeTwoByte)
+        parser.parse_bytes('data', 2)
+
+        return TlsKeyShareEntryInvalidType(parser['group'], parser['data']), parser.parsed_length
+
+    def compose(self):
+        composer = ComposerBinary()
+
+        composer.compose_parsable(self.group)
+        composer.compose_bytes(self.data, 2)
+
+        return composer.composed_bytes
+
+
 class TlsKeyShareEntryVector(VectorParsable):
     @classmethod
     def get_param(cls):
         return VectorParamParsable(
             item_class=TlsKeyShareEntry,
-            fallback_class=None,
+            fallback_class=TlsKeyShareEntryInvalidType,
             min_byte_num=0, max_byte_num=2 ** 16 - 1
         )
 
@@ -727,7 +607,7 @@ class TlsExtensionKeyShareClientHelloRetry(TlsExtensionKeyShareBase):
     def compose(self):
         payload_composer = ComposerBinary()
 
-        payload_composer.compose_parsable(self.selected_group)
+        payload_composer.compose_numeric_enum_coded(self.selected_group)
 
         header_bytes = self._compose_header(payload_composer.composed_length)
 
@@ -922,34 +802,34 @@ class TlsProtocolNameFactory(OpaqueEnumParsable):
         )
 
 
-class TlsProtocolNameList(VectorParsable):
+class TlsProtocolNameList(VectorEnumCodeString):
     @classmethod
     def get_param(cls):
-        return VectorParamParsable(
+        return VectorParamEnumCodeString(
             item_class=TlsProtocolNameFactory,
-            fallback_class=None,
             min_byte_num=2, max_byte_num=2 ** 16 - 1
         )
 
 
 @attr.s
-class TlsExtensionApplicationLayerProtocolNegotiation(TlsExtensionParsed):
+class TlsExtensionApplicationLayerProtocolBase(TlsExtensionParsed):
     protocol_names = attr.ib(
         converter=TlsProtocolNameList,
         validator=attr.validators.instance_of(TlsProtocolNameList),
     )
 
     @classmethod
+    @abc.abstractmethod
     def get_extension_type(cls):
-        return TlsExtensionType.APPLICATION_LAYER_PROTOCOL_NEGOTIATION
+        raise NotImplementedError()
 
     @classmethod
     def _parse(cls, parsable):
-        parser = super(TlsExtensionApplicationLayerProtocolNegotiation, cls)._parse_header(parsable)
+        parser = super(TlsExtensionApplicationLayerProtocolBase, cls)._parse_header(parsable)
 
         parser.parse_parsable('protocol_names', TlsProtocolNameList)
 
-        return TlsExtensionApplicationLayerProtocolNegotiation(parser['protocol_names']), parser.parsed_length
+        return cls(parser['protocol_names']), parser.parsed_length
 
     def compose(self):
         payload_composer = ComposerBinary()
@@ -961,6 +841,18 @@ class TlsExtensionApplicationLayerProtocolNegotiation(TlsExtensionParsed):
         return header_bytes + payload_composer.composed_bytes
 
 
+class TlsExtensionApplicationLayerProtocolNegotiation(TlsExtensionApplicationLayerProtocolBase):
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.APPLICATION_LAYER_PROTOCOL_NEGOTIATION
+
+
+class TlsExtensionApplicationLayerProtocolSettings(TlsExtensionApplicationLayerProtocolBase):
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.APPLICATION_LAYER_PROTOCOL_SETTINGS
+
+
 class TlsExtensionNextProtocolNegotiationClient(TlsExtensionUnusedData):
     @classmethod
     def get_extension_type(cls):
@@ -979,12 +871,11 @@ class TlsNextProtocolNameFactory(OpaqueEnumParsable):
         )
 
 
-class TlsNextProtocolNameList(VectorParsable):
+class TlsNextProtocolNameList(VectorEnumCodeString):
     @classmethod
     def get_param(cls):
-        return VectorParamParsable(
+        return VectorParamEnumCodeString(
             item_class=TlsNextProtocolNameFactory,
-            fallback_class=None,
             min_byte_num=1, max_byte_num=2 ** 16 - 1
         )
 
@@ -1022,6 +913,12 @@ class TlsExtensionNextProtocolNegotiationServer(TlsExtensionParsed):
         return header_composer.composed_bytes + payload_composer.composed_bytes
 
 
+class TlsExtensionChannelId(TlsExtensionUnusedData):
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.CHANNEL_ID
+
+
 class TlsExtensionEncryptThenMAC(TlsExtensionUnusedData):
     @classmethod
     def get_extension_type(cls):
@@ -1034,15 +931,268 @@ class TlsExtensionExtendedMasterSecret(TlsExtensionUnusedData):
         return TlsExtensionType.EXTENDED_MASTER_SECRET
 
 
+@attr.s
+class TlsExtensionShortRecordHeader(TlsExtensionUnusedData):
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.SHORT_RECORD_HEADER
+
+
+class TlsTokenBindingProtocolVersion(ProtocolVersionMajorMinorBase):
+    pass
+
+
+class TlsTokenBindingParamaterFactory(OneByteEnumParsable):
+    @classmethod
+    def get_enum_class(cls):
+        return TlsTokenBindingParamater
+
+    @abc.abstractmethod
+    def compose(self):
+        raise NotImplementedError()
+
+
+class TlsTokenBindingParamaterVector(VectorEnumCodeNumeric):
+    @classmethod
+    def get_param(cls):
+        return VectorParamEnumCodeNumeric(
+            item_class=TlsTokenBindingParamaterFactory,
+            fallback_class=TlsInvalidTypeOneByte,
+            min_byte_num=1,
+            max_byte_num=2 ** 8 - 1,
+        )
+
+
+@attr.s
+class TlsExtensionTokenBinding(TlsExtensionParsed):
+    protocol_version = attr.ib(validator=attr.validators.instance_of(TlsTokenBindingProtocolVersion))
+    parameters = attr.ib(
+        converter=TlsTokenBindingParamaterVector,
+        validator=attr.validators.instance_of(TlsTokenBindingParamaterVector)
+    )
+
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.TOKEN_BINDING
+
+    @classmethod
+    def _parse(cls, parsable):
+        parser = super(TlsExtensionTokenBinding, cls)._parse_header(parsable)
+
+        parser.parse_parsable('protocol_version', TlsTokenBindingProtocolVersion)
+        parser.parse_parsable('parameters', TlsTokenBindingParamaterVector)
+
+        return cls(parser['protocol_version'], parser['parameters']), parser.parsed_length
+
+    def compose(self):
+        payload_composer = ComposerBinary()
+
+        payload_composer.compose_parsable(self.protocol_version)
+        payload_composer.compose_parsable(self.parameters)
+
+        header_bytes = self._compose_header(payload_composer.composed_length)
+
+        return header_bytes + payload_composer.composed_bytes
+
+
+class TlsPskKeyExchangeModeFactory(OneByteEnumParsable):
+    @classmethod
+    def get_enum_class(cls):
+        return TlsPskKeyExchangeMode
+
+    @abc.abstractmethod
+    def compose(self):
+        raise NotImplementedError()
+
+
+class TlsPskKeyExchangeModeVector(VectorEnumCodeNumeric):
+    @classmethod
+    def get_param(cls):
+        return VectorParamEnumCodeNumeric(
+            item_class=TlsPskKeyExchangeModeFactory,
+            fallback_class=TlsInvalidTypeOneByte,
+            min_byte_num=1,
+            max_byte_num=2 ** 8 - 1,
+        )
+
+
+@attr.s
+class TlsExtensionPskKeyExchangeModes(TlsExtensionParsed):
+    key_exchange_modes = attr.ib(
+        converter=TlsPskKeyExchangeModeVector,
+        validator=attr.validators.instance_of(TlsPskKeyExchangeModeVector)
+    )
+
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.PSK_KEY_EXCHANGE_MODES
+
+    @classmethod
+    def _parse(cls, parsable):
+        parser = super(TlsExtensionPskKeyExchangeModes, cls)._parse_header(parsable)
+
+        parser.parse_parsable('key_exchange_modes', TlsPskKeyExchangeModeVector)
+
+        return TlsExtensionPskKeyExchangeModes(parser['key_exchange_modes']), parser.parsed_length
+
+    def compose(self):
+        payload_composer = ComposerBinary()
+
+        payload_composer.compose_parsable(self.key_exchange_modes)
+
+        header_bytes = self._compose_header(payload_composer.composed_length)
+
+        return header_bytes + payload_composer.composed_bytes
+
+
+@attr.s
+class TlsExtensionRecordSizeLimit(TlsExtensionParsed):
+    record_size_limit = attr.ib(validator=attr.validators.instance_of(int))
+
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.RECORD_SIZE_LIMIT
+
+    @classmethod
+    def _parse(cls, parsable):
+        parser = cls._parse_header(parsable)
+
+        parser.parse_numeric('record_size_limit', 2)
+
+        return TlsExtensionRecordSizeLimit(parser['record_size_limit']), parser.parsed_length
+
+    def compose(self):
+        payload_composer = ComposerBinary()
+
+        payload_composer.compose_numeric(self.record_size_limit, 2)
+
+        header_bytes = self._compose_header(payload_composer.composed_length)
+
+        return header_bytes + payload_composer.composed_bytes
+
+
+@attr.s
+class TlsExtensionSignedCertificateTimestampClient(TlsExtensionUnusedData):
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.SIGNED_CERTIFICATE_TIMESTAMP
+
+
+@attr.s
+class TlsExtensionSignedCertificateTimestampServer(TlsExtensionParsed):
+    scts = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(SignedCertificateTimestampList)))
+
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.SIGNED_CERTIFICATE_TIMESTAMP
+
+    @classmethod
+    def _parse(cls, parsable):
+        parser = super(cls, cls)._parse_header(parsable)
+
+        parser.parse_parsable('scts', SignedCertificateTimestampList)
+
+        return cls(parser['scts']), parser.parsed_length
+
+    def compose(self):
+        payload_composer = ComposerBinary()
+
+        payload_composer.compose_parsable(self.scts)
+
+        header_bytes = self._compose_header(payload_composer.composed_length)
+
+        return header_bytes + payload_composer.composed_bytes
+
+
+class TlsCertificateCompressionAlgorithmFactory(TwoByteEnumParsable):
+    @classmethod
+    def get_enum_class(cls):
+        return TlsCertificateCompressionAlgorithm
+
+    @abc.abstractmethod
+    def compose(self):
+        raise NotImplementedError()
+
+
+class TlsCertificateCompressionAlgorithmVector(VectorEnumCodeNumeric):
+    @classmethod
+    def get_param(cls):
+        return VectorParamEnumCodeNumeric(
+            item_class=TlsCertificateCompressionAlgorithmFactory,
+            fallback_class=TlsInvalidTypeTwoByte,
+            min_byte_num=2, max_byte_num=2 ** 8 - 2
+        )
+
+
+@attr.s
+class TlsExtensionCompressCertificate(TlsExtensionParsed):
+    compression_algorithms = attr.ib(
+        converter=TlsCertificateCompressionAlgorithmVector,
+        validator=attr.validators.instance_of(TlsCertificateCompressionAlgorithmVector),
+    )
+
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.COMPRESS_CERTIFICATE
+
+    @classmethod
+    def _parse(cls, parsable):
+        parser = super(TlsExtensionCompressCertificate, cls)._parse_header(parsable)
+
+        parser.parse_parsable('compression_algorithms', TlsCertificateCompressionAlgorithmVector)
+
+        return cls(parser['compression_algorithms']), parser.parsed_length
+
+    def compose(self):
+        payload_composer = ComposerBinary()
+
+        payload_composer.compose_parsable(self.compression_algorithms)
+
+        header_bytes = self._compose_header(payload_composer.composed_length)
+
+        return header_bytes + payload_composer.composed_bytes
+
+
+@attr.s
+class TlsExtensionPadding(TlsExtensionParsed):
+    length = attr.ib(validator=attr.validators.instance_of(six.integer_types))
+
+    @classmethod
+    def get_extension_type(cls):
+        return TlsExtensionType.PADDING
+
+    @classmethod
+    def _parse(cls, parsable):
+        parser = super(TlsExtensionPadding, cls)._parse_header(parsable)
+
+        parser.parse_raw('padding', parser['extension_length'])
+        try:
+            non_zero_int = next(byte for byte in six.iterbytes(parser['padding']) if byte != 0)
+            raise InvalidValue(six.int2byte(non_zero_int), cls)
+        except StopIteration:
+            pass
+
+        return cls(parser['extension_length']), parser.parsed_length
+
+    def compose(self):
+        payload_composer = ComposerBinary()
+
+        payload_composer.compose_raw(self.length * b'\x00')
+
+        header_bytes = self._compose_header(payload_composer.composed_length)
+
+        return header_bytes + payload_composer.composed_bytes
+
+
 class TlsExtensionVariantBase(VariantParsable):
     @classmethod
     @abc.abstractmethod
-    def _get_parsed_extensions(cls):
+    def get_parsed_extensions(cls):
         raise NotImplementedError()
 
     @classmethod
     def _get_variants(cls):
-        variants = cls._get_parsed_extensions()
+        variants = cls.get_parsed_extensions()
 
         variants.update([
             (extension_type, (TlsExtensionUnparsed, ))
@@ -1055,36 +1205,54 @@ class TlsExtensionVariantBase(VariantParsable):
 
 class TlsExtensionVariantClient(TlsExtensionVariantBase):
     @classmethod
-    def _get_parsed_extensions(cls):
+    def get_parsed_extensions(cls):
         return collections.OrderedDict([
             (TlsExtensionType.APPLICATION_LAYER_PROTOCOL_NEGOTIATION,
                 [TlsExtensionApplicationLayerProtocolNegotiation, ]),
+            (TlsExtensionType.APPLICATION_LAYER_PROTOCOL_SETTINGS,
+                [TlsExtensionApplicationLayerProtocolSettings, ]),
+            (TlsExtensionType.CHANNEL_ID, [TlsExtensionChannelId, ]),
+            (TlsExtensionType.COMPRESS_CERTIFICATE, [TlsExtensionCompressCertificate, ]),
             (TlsExtensionType.ENCRYPT_THEN_MAC, [TlsExtensionEncryptThenMAC, ]),
             (TlsExtensionType.EXTENDED_MASTER_SECRET, [TlsExtensionExtendedMasterSecret, ]),
             (TlsExtensionType.RENEGOTIATION_INFO, [TlsExtensionRenegotiationInfo, ]),
             (TlsExtensionType.NEXT_PROTOCOL_NEGOTIATION, [TlsExtensionNextProtocolNegotiationClient, ]),
-            (TlsExtensionType.SERVER_NAME, [TlsExtensionServerName, ]),
+            (TlsExtensionType.PADDING, [TlsExtensionPadding, ]),
+            (TlsExtensionType.SERVER_NAME, [TlsExtensionServerNameClient, ]),
             (TlsExtensionType.SESSION_TICKET, [TlsExtensionSessionTicket, ]),
+            (TlsExtensionType.STATUS_REQUEST, [TlsExtensionCertificateStatusRequest, ]),
             (TlsExtensionType.SUPPORTED_GROUPS, [TlsExtensionEllipticCurves, ]),
+            (TlsExtensionType.DELEGATED_CREDENTIALS, [TlsExtensionDelegatedCredentials, ]),
             (TlsExtensionType.EC_POINT_FORMATS, [TlsExtensionECPointFormats, ]),
             (TlsExtensionType.KEY_SHARE, [TlsExtensionKeyShareClient, ]),
+            (TlsExtensionType.KEY_SHARE_RESERVED, [TlsExtensionKeyShareReservedClient, ]),
+            (TlsExtensionType.PSK_KEY_EXCHANGE_MODES, [TlsExtensionPskKeyExchangeModes, ]),
+            (TlsExtensionType.RECORD_SIZE_LIMIT, [TlsExtensionRecordSizeLimit, ]),
+            (TlsExtensionType.SHORT_RECORD_HEADER, [TlsExtensionShortRecordHeader, ]),
             (TlsExtensionType.SIGNATURE_ALGORITHMS, [TlsExtensionSignatureAlgorithms, ]),
+            (TlsExtensionType.SIGNATURE_ALGORITHMS_CERT, [TlsExtensionSignatureAlgorithmsCert, ]),
+            (TlsExtensionType.SIGNED_CERTIFICATE_TIMESTAMP, [TlsExtensionSignedCertificateTimestampClient, ]),
             (TlsExtensionType.SUPPORTED_VERSIONS, [TlsExtensionSupportedVersionsClient, ]),
+            (TlsExtensionType.TOKEN_BINDING, [TlsExtensionTokenBinding, ]),
         ])
 
 
 class TlsExtensionVariantServer(TlsExtensionVariantBase):
     @classmethod
-    def _get_parsed_extensions(cls):
+    def get_parsed_extensions(cls):
         return collections.OrderedDict([
             (TlsExtensionType.APPLICATION_LAYER_PROTOCOL_NEGOTIATION,
                 [TlsExtensionApplicationLayerProtocolNegotiation, ]),
+            (TlsExtensionType.CHANNEL_ID, [TlsExtensionChannelId, ]),
             (TlsExtensionType.EC_POINT_FORMATS, [TlsExtensionECPointFormats, ]),
             (TlsExtensionType.ENCRYPT_THEN_MAC, [TlsExtensionEncryptThenMAC, ]),
             (TlsExtensionType.EXTENDED_MASTER_SECRET, [TlsExtensionExtendedMasterSecret, ]),
             (TlsExtensionType.KEY_SHARE, [TlsExtensionKeyShareClientHelloRetry, TlsExtensionKeyShareServer]),
             (TlsExtensionType.NEXT_PROTOCOL_NEGOTIATION, [TlsExtensionNextProtocolNegotiationServer, ]),
+            (TlsExtensionType.RECORD_SIZE_LIMIT, [TlsExtensionRecordSizeLimit, ]),
             (TlsExtensionType.RENEGOTIATION_INFO, [TlsExtensionRenegotiationInfo, ]),
+            (TlsExtensionType.SERVER_NAME, [TlsExtensionServerNameServer, ]),
             (TlsExtensionType.SESSION_TICKET, [TlsExtensionSessionTicket, ]),
+            (TlsExtensionType.SIGNED_CERTIFICATE_TIMESTAMP, [TlsExtensionSignedCertificateTimestampServer, ]),
             (TlsExtensionType.SUPPORTED_VERSIONS, [TlsExtensionSupportedVersionsServer, ]),
         ])
